@@ -1105,15 +1105,27 @@ k_gather <- function(reference, indices) {
 #'
 #' @export
 k_get_session <- function() {
-  keras$backend$get_session()
+  
+  if(tensorflow::tf_version() >= '2.0'){
+    warning("Tensorflow 2.0 does not expose the 'k_get_session()' directly any more. Instead use 'tf$compat$v1$keras$backend$get_session()'", call. = FALSE)
+    tensorflow::tf$compat$v1$keras$backend$get_session()
+  }
+  else
+    keras$backend$get_session()
 }
 
 #' @rdname k_get_session
 #' @export
 k_set_session <- function(session) {
-  keras$backend$set_session(
-    session = session
-  )
+
+  if(tensorflow::tf_version() >= '2.0'){
+    warning("Tensorflow 2.0 does not expose the 'k_set_session()' directly any more. Instead use 'tf$compat$v1$keras$backend$set_session()'", call. = FALSE)
+    tensorflow::tf$compat$v1$keras$backend$set_session(session = session)
+  }
+  else
+    keras$backend$set_session(
+      session = session
+    )
 }
 
 
@@ -1601,6 +1613,10 @@ k_log <- function(x) {
 #'
 #' @export
 k_logsumexp <- function(x, axis = NULL, keepdims = FALSE) {
+  
+  if (tensorflow::tf_version() >= "2.2")
+    stop("k_logsumexp is deprecated. use tensorflow::tf$reduce_logsumexp ")
+  
   keras$backend$logsumexp(
     x = x,
     axis = as_axis(axis),
