@@ -8,7 +8,9 @@ if (os.getenv('KERAS_IMPLEMENTATION', 'tensorflow') == 'keras'):
 else:
   from tensorflow.keras.layers import Layer
   def shape_filter(shape):
-    if not isinstance(shape, list):
+    if isinstance(shape, tuple):
+      return list(shape)
+    elif not isinstance(shape, list):
       return shape.as_list()
     else:
       return shape
