@@ -1,7 +1,7 @@
 
 context("generators")
 
-source("utils.R")
+
 
 test_succeeds("image data generator can be used for training", {
 
@@ -49,32 +49,35 @@ test_succeeds("image data generator can be used for training", {
   datagen %>% fit_image_data_generator(X_train)
   
   # train using generator
-  x <- capture_output(
+  expect_warning_if(tensorflow::tf_version() >= "2.1", {
+    
+    x <- capture_output(
+      model %>%
+        fit_generator(flow_images_from_data(X_train, Y_train, datagen, batch_size = 32),
+                      steps_per_epoch = 32, epochs = 2, verbose = 0)  
+    )
+    
+    # evaluate using generator
+    scores <- model %>%
+      evaluate_generator(flow_images_from_data(X_test, Y_test, datagen, batch_size = 32),
+                         steps = 5)
+    
+    expect_true(all(names(scores) %in% c("loss", "acc", "accuracy")))
+    
+    # predict using generator
     model %>%
-      fit_generator(flow_images_from_data(X_train, Y_train, datagen, batch_size = 32),
-                    steps_per_epoch = 32, epochs = 2, verbose = 0)  
-  )
-  
-  # evaluate using generator
-  scores <- model %>%
-    evaluate_generator(flow_images_from_data(X_test, Y_test, datagen, batch_size = 32),
-                       steps = 5)
-  
-  expect_true(all(names(scores) %in% c("loss", "acc", "accuracy")))
-  
-  # predict using generator
-  model %>%
-    predict_generator(flow_images_from_data(X_test, Y_test, datagen, batch_size = 32),
-                      steps = 5)
+      predict_generator(flow_images_from_data(X_test, Y_test, datagen, batch_size = 32),
+                        steps = 5)
+    
+  })
 })
 
 test_succeeds("R function can be used as custom generator", {
  
-  if (tensorflow::tf_version() >= "2.1")
-    skip("TODO: R based generators are not working with TF >= 2.1")
+  if (tensorflow::tf_version() == "2.1")
+    skip("don't work in tf2.1")
   
   # create model
-  library(keras)
   model <- keras_model_sequential()
   
   # add layers and compile the model
@@ -106,26 +109,30 @@ test_succeeds("R function can be used as custom generator", {
   }
   
   # Train the model, iterating on the data in batches of 32 samples
-  model %>% 
-    fit_generator(sampling_generator(X_train, Y_train, batch_size = 32), 
-                  steps_per_epoch = 10, epochs = 2, verbose = 0)
   
-  # Evaluate the model
-  model %>% 
-    evaluate_generator(sampling_generator(X_test, Y_test, batch_size = 32), 
-                       steps = 10)
+  expect_warning_if(tensorflow::tf_version() >= "2.1", {
+    model %>% 
+      fit_generator(sampling_generator(X_train, Y_train, batch_size = 32), 
+                    steps_per_epoch = 10, epochs = 2, verbose = 1)
+    
+    # Evaluate the model
+    model %>% 
+      evaluate_generator(sampling_generator(X_test, Y_test, batch_size = 32), 
+                         steps = 10)
+    
+    # generate predictions
+    model %>% 
+      predict_generator(sampling_generator(X_test, batch_size = 32), 
+                        steps = 10)
+  })
   
-  # generate predictions
-  model %>% 
-    predict_generator(sampling_generator(X_test, batch_size = 32), 
-                      steps = 10)
    
 })
 
 test_succeeds("R function can be used as custom generator with multiple inputs", {
   
-  if (tensorflow::tf_version() >= "2.1")
-    skip("TODO: R based generators are not working with TF >= 2.1")
+  if (tensorflow::tf_version() == "2.1")
+    skip("don't work in tf2.1")
   
   input1 <- layer_input(shape = 1)
   input2 <- layer_input(shape = 1)
@@ -140,16 +147,18 @@ test_succeeds("R function can be used as custom generator with multiple inputs",
   }
   
   model %>% compile(loss = "mse", optimizer = "sgd")
+  expect_warning_if(tensorflow::tf_version() > "2.1", {
+    model %>% fit_generator(generator, steps_per_epoch = 10, 
+                            validation_data = generator, validation_steps = 2,
+                            verbose = 1)  
+  })
   
-  model %>% fit_generator(generator, steps_per_epoch = 10, 
-                          validation_data = generator, validation_steps = 2,
-                          verbose = 0)
 })
 
 test_succeeds("Fixed validation_data instead of generator with fit_generator", {
 
-  if (tensorflow::tf_version() >= "2.1")
-    skip("TODO: R based generators are not working with TF >= 2.1")
+  if (tensorflow::tf_version() == "2.1")
+    skip("don't work in tf2.1")
   
   input1 <- layer_input(shape = 1)
   input2 <- layer_input(shape = 1)
@@ -165,17 +174,19 @@ test_succeeds("Fixed validation_data instead of generator with fit_generator", {
   
   model %>% compile(loss = "mse", optimizer = "sgd")
   
-  model %>% fit_generator(
-    generator, steps_per_epoch = 2, 
-    validation_data = list(list(1, 2), 3),
-    verbose = 0)
+  expect_warning_if(tensorflow::tf_version() >= "2.1", {
+    model %>% fit_generator(
+      generator, steps_per_epoch = 2, 
+      validation_data = list(list(1, 2), 3),
+      verbose = 0)  
+  })
   
 })
 
 test_succeeds("Can use a custom preprocessing function in image_data_generator", {
   
-  if (tensorflow::tf_version() >= "2.1")
-    skip("TODO: R based generators are not working with TF >= 2.1")
+  if (tensorflow::tf_version() == "2.1")
+    skip("don't work in tf2.1")
   
   img_gen <- image_data_generator(preprocessing_function = function(x) x/255)
   
@@ -194,10 +205,13 @@ test_succeeds("Can use a custom preprocessing function in image_data_generator",
   model %>% compile(loss = "categorical_crossentropy", optimizer = "adam", metrics = "accuracy")
   
   # test fitting the model
-  model %>% fit_generator(flow, steps_per_epoch = 5, epochs = 1, verbose = 0)
-  preds <- predict_generator(model, flow, steps = 5)
-  eval <- evaluate_generator(model, flow, steps = 10)
-
+  expect_warning_if(tensorflow::tf_version() >= "2.1", {
+    model %>% fit_generator(flow, steps_per_epoch = 5, epochs = 1, verbose = 0)
+    preds <- predict_generator(model, flow, steps = 5)
+    eval <- evaluate_generator(model, flow, steps = 10)  
+  })
+  
 })
 
 
+
