@@ -1,6 +1,6 @@
 context("losses")
 
-source("utils.R")
+
 
 test_loss <- function(name, test_direct_call = TRUE) {
   
@@ -38,5 +38,19 @@ test_loss("kullback_leibler_divergence")
 test_loss("poisson")
 test_loss("cosine_similarity", test_direct_call = FALSE)
 
-
+test_succeeds("binary_crossentropy new args", {
+  
+  y_true <- k_constant(matrix(runif(100), nrow = 10, ncol = 10))
+  y_pred <- k_constant(matrix(runif(100), nrow = 10, ncol = 10))
+  
+  if (tensorflow::tf_version() >= "2.2") {
+    out <- loss_binary_crossentropy(y_true, y_pred, from_logits = TRUE, label_smoothing = 0.5)  
+  } else {
+    expect_warning(
+      out <- loss_binary_crossentropy(y_true, y_pred, from_logits = TRUE, label_smoothing = 0.5)    
+    )
+  }
+  
+  expect_equal(out$shape$as_list(),10)
+})
 
