@@ -1,7 +1,5 @@
 context("preprocessing")
 
-source("utils.R")
-
 test_call_succeeds("pad_sequences", {
   a <- list(list(1), list(1,2), list(1,2,3))
   b <- pad_sequences(a, maxlen=3, padding='pre')    
@@ -28,6 +26,18 @@ test_call_succeeds("text_hashing_trick", required_version = "2.0.5", {
   expect_equal(length(encoded), 6)
 })
 
+test_call_succeeds("missing text data", required_version = "2.0.5", {
+  expect_error(text_hashing_trick(letters, 10),
+               "`text` should be length 1")
+  
+  texts <- c(
+    'Dogs and cats living together.',
+    NA_character_
+  )
+  expect_true(all(!is.na(text_hashing_trick(texts[1], 10))))
+  expect_true(all( is.na(text_hashing_trick(texts[2], 10))))
+})
+
 test_succeeds("use of text tokenizer", {
   texts <- c(
     'The cat sat on the mat.',
@@ -90,18 +100,33 @@ test_succeeds("flow images from dataframe works", {
       class = letters[1:10], 
       stringsAsFactors = FALSE
       )
+    
     img_gen <- flow_images_from_dataframe(
       df, 
       directory = ".", 
       x_col = "fname", 
-      y_col = "class", 
-      drop_duplicates = FALSE
-      )
+      y_col = "class"
+    )
     
     batch <- reticulate::iter_next(img_gen)
     
     expect_equal(dim(batch[[1]]), c(10, 256, 256, 3))
     expect_equal(dim(batch[[2]]), c(10, 10))
+    
+    if (tensorflow::tf_version() >= "2.3") {
+      
+      expect_warning(
+        img_gen <- flow_images_from_dataframe(
+          df, 
+          directory = ".", 
+          x_col = "fname", 
+          y_col = "class", 
+          drop_duplicates = TRUE
+        )
+      )
+      
+    }
+    
   }
 })
 
@@ -110,7 +135,8 @@ test_succeeds("flow images from directory works", {
   if (!have_pillow())
     skip("Pillow required.")
   
-  dir <- tempdir()
+  dir <- tempfile()
+  dir.create(dir)
   dir.create(paste0(dir, "/flow-img"))
   dir <- paste0(dir, "/flow-img")
   dir.create(paste0(dir, "/0"))
@@ -140,14 +166,44 @@ test_succeeds("flow images from directory works", {
   
   model %>% compile(loss = "binary_crossentropy", optimizer = "adam")
   
-  # test fitting the model
-  model %>% fit_generator(gen, steps_per_epoch = 20)
-  
-  # test predictions
-  preds <- predict_generator(model, gen, steps = 10)
-  
-  # evaluate
-  eva <- evaluate_generator(model, gen, steps = 10)
+  expect_warning_if(tensorflow::tf_version() >= "2.1", {
+    # test fitting the model
+    model %>% fit_generator(gen, steps_per_epoch = 20)
+    
+    # test predictions
+    preds <- predict_generator(model, gen, steps = 10)
+    
+    # evaluate
+    eva <- evaluate_generator(model, gen, steps = 10)
+  })
 })
 
+test_succeeds("images_dataset_from_directory", {
+  
+  if (tensorflow::tf_version() < "2.3")
+    skip("requires tf_version() >= 2.3")
+  
+  dir <- tempfile()
+  dir.create(dir)
+  dir.create(file.path(dir, "0"))
+  dir.create(file.path(dir, "1"))
+  
+  mnist <- dataset_mnist()
+  ind <- which(mnist$train$y %in% c(0, 1))
+  
+  for (i in ind) {
+    img <- mnist$train$x[i,,]/255
+    rname <- paste(sample(letters, size = 10, replace = TRUE), collapse = "")
+    jpeg::writeJPEG(img, target = paste0(dir, "/", mnist$train$y[i], "/", rname, ".jpeg"))
+  }
+  
+  data <- image_dataset_from_directory(dir)
+  
+  iter <- reticulate::as_iterator(data)
+  d <- reticulate::iter_next(iter)
+  
+  expect_equal(d[[1]]$shape$as_list(), c(32, 256, 256, 3))
+  expect_equal(d[[2]]$shape$as_list(), c(32))
+  
+})
 
